# Copyright (c) 2016, Camptocamp SA
# All rights reserved.

# Auto-generated file. Do not Edit!
INFO = {
    "git_hash": "bba89f318a678c01900b21ed12123bbdfb0d10b5",
    "version": "2.0",
    "git_tag": "2.1.0.dev4",
}
